package org.gcube.portlets.user.td.expressionwidget.shared.replace;

import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_Concat;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_SubstringByIndex;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_SubstringByRegex;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_SubstringPosition;
import org.gcube.portlets.user.td.expressionwidget.shared.model.composite.text.C_TextReplaceMatchingRegex;
import org.gcube.portlets.user.td.expressionwidget.shared.model.leaf.C_ColumnReferencePlaceholder;
import org.gcube.portlets.user.td.expressionwidget.shared.model.leaf.C_TypedColumnReference;
import org.gcube.portlets.user.td.expressionwidget.shared.model.leaf.TD_Value;
import org.gcube.portlets.user.td.gwtservice.shared.tr.ColumnData;
import org.gcube.portlets.user.td.widgetcommonevent.shared.expression.C_Expression;
import org.gcube.portlets.user.td.widgetcommonevent.shared.tr.column.ColumnDataType;
import org.gcube.portlets.user.td.widgetcommonevent.shared.tr.column.ColumnTypeMap;

import com.allen_sauer.gwt.log.client.Log;

/**
 * OperatorTypeMap creates a C_Expression usable client-side
 * 
 * @author "Giancarlo Panichi" <a
 *         href="mailto:g.panichi@isti.cnr.it">g.panichi@isti.cnr.it</a>
 * 
 */
public class ReplaceTypeMap {
	private String readableExpression;

	public ReplaceTypeMap() {

	}
	
	public C_Expression map(ColumnData column, ReplaceType replaceType,
			String firstArg, String secondArg) {
		return map(column, replaceType,firstArg,  secondArg, null, null, null, null);
	}

	public C_Expression map(ColumnData column, ReplaceType replaceType,
			String firstArg, String secondArg, C_Expression firstE,
			C_Expression secondE, String readableFirstE, String readableSecondE) {
		Log.debug("ReplaceTypeMap Map");
		C_Expression exp = null;
		readableExpression = new String();
		if (column.getTrId() == null) {
			exp = mapPlaceHolder(column, replaceType, firstArg, secondArg,
					firstE, secondE, readableFirstE,readableSecondE);
		} else {
			exp = mapTypedColumnReference(column, replaceType, firstArg,
					secondArg, firstE, secondE,readableFirstE,readableSecondE);
		}
		return exp;
	}

	public C_Expression mapPlaceHolder(ColumnData column,
			ReplaceType replaceType, String firstArg, String secondArg,
			C_Expression firstE, C_Expression secondE,String readableFirstE, String readableSecondE) {
		Log.debug("OperatorTypeMap Map Place Holder");
		C_Expression exp = null;
		Log.debug("Column Data Type Name:" + column.getDataTypeName());
		ColumnDataType dataType = ColumnTypeMap.getColumnDataType(column
				.getDataTypeName());
		Log.debug("Data Type:" + dataType);
		C_ColumnReferencePlaceholder placeHolder = new C_ColumnReferencePlaceholder(
				dataType, column.getColumnId());
		Log.debug("placeHolder:" + placeHolder);
		String readablePlaceHolder = column.getColumnId();
		TD_Value fArg, sArg;
		C_Expression posFrom, posTo;

		switch (replaceType) {
		case Null:
			exp = new TD_Value(ColumnDataType.Text, "");
			readableExpression = "Null()";
			break;
		case Value:
			exp = new TD_Value(dataType, firstArg);
			readableExpression = "Value(" + firstArg + ")";
			break;
		case ColumnValue:
			exp = placeHolder;
			readableExpression = "ColumnValue(" + readablePlaceHolder + ")";
			break;
		case Concat:
			exp = new C_Concat(firstE, secondE);
			readableExpression = "Concat(" + readableFirstE
					+ ", " + readableSecondE + ")";
			break;
		case SubstringByRegex:
			fArg = new TD_Value(ColumnDataType.Text, firstArg);
			exp = new C_SubstringByRegex(placeHolder, fArg);
			readableExpression = "SubStringByRegex(" + readablePlaceHolder
					+ ", " + firstArg + ")";
			break;
		case SubstringByIndex:
			fArg = new TD_Value(ColumnDataType.Integer, firstArg);
			sArg = new TD_Value(ColumnDataType.Integer, secondArg);
			exp = new C_SubstringByIndex(placeHolder, fArg, sArg);
			readableExpression = "SubstringByIndex(" + readablePlaceHolder
					+ ", " + firstArg + ", " + secondArg + ")";
			break;
		case SubstringByCharSeq:
			fArg = new TD_Value(ColumnDataType.Text, firstArg);
			posFrom = new C_SubstringPosition(placeHolder, fArg);
			sArg = new TD_Value(ColumnDataType.Text, secondArg);
			posTo = new C_SubstringPosition(placeHolder, fArg);
			exp = new C_SubstringByIndex(placeHolder, posFrom, posTo);
			readableExpression = "SubstringByCharSeq(" + readablePlaceHolder
					+ ", " + firstArg + ", " + secondArg + ")";
			break;
		case TextReplaceMatchingRegex:
			fArg = new TD_Value(ColumnDataType.Text, firstArg);
			sArg = new TD_Value(ColumnDataType.Text, secondArg);
			exp = new C_TextReplaceMatchingRegex(placeHolder, fArg, sArg);
			readableExpression = "TextReplaceMatchingRegex(" + readablePlaceHolder
					+ ", " + firstArg + ", " + secondArg + ")";
			break;
		default:
			break;
		}
		return exp;
	}

	public C_Expression mapTypedColumnReference(ColumnData column,
			ReplaceType replaceType, String firstArg, String secondArg,
			C_Expression firstE, C_Expression secondE, String readableFirstE, String readableSecondE) {
		Log.debug("OperatorTypeMap Map Typed Column Reference");
		C_Expression exp = null;

		Log.debug("Column Data Type Name:" + column.getDataTypeName());
		ColumnDataType dataType = ColumnTypeMap.getColumnDataType(column
				.getDataTypeName());
		Log.debug("Data Type:" + dataType);
		C_TypedColumnReference columnReference = new C_TypedColumnReference(
				column.getTrId(), dataType, column.getColumnId());
		Log.debug("Typed Column Reference:" + columnReference);
		String readableColumnReference = column.getLabel();

		TD_Value fArg, sArg;
		C_Expression posFrom, posTo;

		switch (replaceType) {
		case Null:
			exp = new TD_Value(ColumnDataType.Text, "");
			readableExpression = "Null()";
			break;
		case Value:
			exp = new TD_Value(dataType, firstArg);
			readableExpression = "Value(" + firstArg + ")";
			break;
		case ColumnValue:
			exp = columnReference;
			readableExpression = "ColumnValue(" + readableColumnReference + ")";
			break;
		case Concat:
			exp = new C_Concat(firstE, secondE);
			readableExpression = "Concat(" + readableFirstE
					+ ", " + readableSecondE + ")";
			break;
		case SubstringByRegex:
			fArg = new TD_Value(ColumnDataType.Text, firstArg);
			exp = new C_SubstringByRegex(columnReference, fArg);
			readableExpression = "SubStringByRegex(" + readableColumnReference
					+ ", " + firstArg + ")";
			break;
		case SubstringByIndex:
			fArg = new TD_Value(ColumnDataType.Integer, firstArg);
			sArg = new TD_Value(ColumnDataType.Integer, secondArg);
			exp = new C_SubstringByIndex(columnReference, fArg, sArg);
			readableExpression = "SubstringByIndex(" + readableColumnReference
					+ ", " + firstArg + ", " + secondArg + ")";
			break;
		case SubstringByCharSeq:
			fArg = new TD_Value(ColumnDataType.Text, firstArg);
			posFrom = new C_SubstringPosition(columnReference, fArg);
			sArg = new TD_Value(ColumnDataType.Text, secondArg);
			posTo = new C_SubstringPosition(columnReference, fArg);
			exp = new C_SubstringByIndex(columnReference, posFrom, posTo);
			readableExpression = "SubstringByCharSeq("
					+ readableColumnReference + ", " + firstArg + ", "
					+ secondArg + ")";
			break;
		case TextReplaceMatchingRegex:
			fArg = new TD_Value(ColumnDataType.Text, firstArg);
			sArg = new TD_Value(ColumnDataType.Text, secondArg);
			exp = new C_TextReplaceMatchingRegex(columnReference, fArg, sArg);
			readableExpression = "TextReplaceMatchingRegex(" + readableColumnReference
					+ ", " + firstArg + ", " + secondArg + ")";
			break;
		default:
			break;
		}

		return exp;
	}

	public String getReadableExpression() {
		return readableExpression;
	}

}
